<?php
    $timelines = carbon_get_post_meta( get_the_ID(), 'timeline_items' );
?>
<div class="page timeline-horizontal-7 timeline-post-<?php echo esc_attr(get_the_ID()); ?>" id="horizontal-basic">
    <div class="views">
    	<?php foreach ( $timelines as $timeline ) { ?>
        <div class="view view--1">
            <div class="view__inner">
                <h1 class="timeline-title"><?php echo esc_html($timeline['timeline_item_title']); ?></h1>
                <h2 class="timeline-info"><?php echo esc_html($timeline['timeline_date']); ?></h2>
                <p class="timeline-text"><?php echo esc_html($timeline['timeline_item_content']); ?></p>
            </div>
        </div>
    	<?php } ?>
    </div>
    <div class="timeline__wrapper">
        <div class="timeline-7">
            <ul class="timeline__list">
    		<?php $no = 0; foreach ( $timelines as $timeline ) { $no++; ?>
                <li class="timeline__item <?php if($no == 1) {echo esc_attr('timeline__item--active');} ?>">
                	<a class="timeline__link" href="#">
                        <div class="timeline__item__point"></div>
                    </a>
                    <div class="timeline__item__content">
                        <div class="timeline__item__shadow"></div>
                    </div>
                    <div class="timeline__item__year"><?php echo esc_html($timeline['timeline_date']); ?></div>
                </li>
    		<?php } ?>
            </ul>
            <div class="timeline__path">
                <div class="timeline__path__triangle timeline__path__triangle--moving"></div>
                <div class="timeline__path__triangle timeline__path__triangle--static-1"></div>
                <div class="timeline__path__triangle timeline__path__triangle--static-2"></div>
                <div class="timeline__path__triangle timeline__path__triangle--static-3"></div>
            </div>
        </div>
    </div>
</div>
<script>
jQuery(document).ready(function($){
	let areClipPathShapesSupported = function() {

        let base = 'clipPath',
            prefixes = ['webkit', 'moz', 'ms', 'o'],
            properties = [base],
            testElement = document.createElement('testelement'),
            attribute = 'polygon(50% 0%, 0% 100%, 100% 100%)';

        // Push the prefixed properties into the array of properties.
        for (let i = 0, l = prefixes.length; i < l; i++) {
            if (window.CP.shouldStopExecution(0)) break;
            let prefixedProperty = prefixes[i] + base.charAt(0).toUpperCase() + base.slice(1); // remember to capitalize!
            properties.push(prefixedProperty);
        }

        // Interate over the properties and see if they pass two tests.
        window.CP.exitedLoop(0);
        for (let i = 0, l = properties.length; i < l; i++) {
            if (window.CP.shouldStopExecution(1)) break;
            let property = properties[i];

            // First, they need to even support clip-path (IE <= 11 does not)...
            if (testElement.style[property] === '') {

                // Second, we need to see what happens when we try to create a CSS shape...
                testElement.style[property] = attribute;
                if (testElement.style[property] !== '') {
                    return true;
                }
            }
        }
        window.CP.exitedLoop(1);

        return false;
    };

    $(() => {
        let defaultSlickSpeed = 300;

        $('.views').
        slick({
            speed: defaultSlickSpeed
        }).

        on('beforeChange', (evt, slick, currentSlide, nextSlide) => {
            let delta = Math.abs(currentSlide - nextSlide);
            if (delta === slick.slideCount - 1) {
                delta = 1;
            }
            $('.views').slick('slickSetOption', 'speed', delta * defaultSlickSpeed);
        }).
        on('afterChange', (evt, slick, currentSlide) => {
            $('.views').slick('slickSetOption', 'speed', defaultSlickSpeed);

            $('.timeline__list').find('.timeline__item--active').removeClass('timeline__item--active');
            $('.timeline__list').find(`.timeline__item:nth-child(${currentSlide + 1})`).addClass('timeline__item--active');
        });

        $('.timeline__link').on('click', evt => {
            evt.preventDefault();
            $('.timeline__item--active').removeClass('timeline__item--active');
            $(evt.currentTarget).parent().addClass('timeline__item--active');

            $('.views').slick('slickGoTo', $(evt.currentTarget).parent().prevAll('li').length);

        });

        let timelineOffset = $('.timeline-7').offset().left;
        let triangleWidth = $('.timeline__path__triangle--moving').outerWidth();

        $('.timeline-7').on('mousemove', evt => {
            let value = evt.pageX - timelineOffset - triangleWidth / 2;
            $('.timeline__path__triangle--moving').css({
                transform: `translateX(${value}px)`
            });

        });

        if (!areClipPathShapesSupported()) {
            $('body').addClass('no-clippath');
        } else {
            $('body').addClass('clippath');
        }

    });

});

</script>